﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Problems.BinPacking.Shapes;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Problems.BinPacking.PackingBin;

namespace HeuristicLab.Problems.BinPacking.PackingItem {
  [Item("RectangularPackingItem", "Represents a rectangular packing-item for bin-packing problems.")]
  [StorableClass]
  public class RectangularPackingItem : RectangularPackingShape, IPackingItem {

    public RectangularPackingBin TargetBin {
      get { return ((IValueParameter<RectangularPackingBin>)Parameters["TargetBin"]).Value; }
      set { ((IValueParameter<RectangularPackingBin>)Parameters["TargetBin"]).Value = value; }
    }

    public double Weight {
      get { return ((IFixedValueParameter<DoubleValue>)Parameters["Weight"]).Value.Value; }
      set { ((IFixedValueParameter<DoubleValue>)Parameters["Weight"]).Value.Value = value; }
    }

    public int Material {
      get { return ((IFixedValueParameter<IntValue>)Parameters["Material"]).Value.Value; }
      set { ((IFixedValueParameter<IntValue>)Parameters["Material"]).Value.Value = value; }
    }

    [StorableConstructor]
    protected RectangularPackingItem(bool deserializing) : base(deserializing) { }
    protected RectangularPackingItem(RectangularPackingItem original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RectangularPackingItem(this, cloner);
    }

    public RectangularPackingItem()
      : base() {
      Parameters.Add(new ValueParameter<RectangularPackingBin>("TargetBin"));
      Parameters.Add(new FixedValueParameter<DoubleValue>("Weight"));
      Parameters.Add(new FixedValueParameter<IntValue>("Material"));
    }

    public RectangularPackingItem(int width, int height, RectangularPackingBin targetBin)
      : this() {
      this.Weight = width;
      this.Height = height;
      this.TargetBin = (RectangularPackingBin)targetBin.Clone();
    }

    public bool SupportsStacking(IPackingItem other) {
      return ((other.Material < this.Material) || (other.Material.Equals(this.Material) && other.Weight <= this.Weight));
    }

    public void AddTargetBinMeasures(int[] targetBinMeasures) {
      TargetBin = new RectangularPackingBin(targetBinMeasures[0], targetBinMeasures[1]);
    }
  }
}
